package bicliqueviz;

import edu.uci.ics.jung.visualization.VisualizationViewer;
import edu.uci.ics.jung.algorithms.layout.FRLayout;
import edu.uci.ics.jung.graph.util.Pair;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.UndirectedSparseGraph;
import java.awt.Color;
import java.awt.Dimension;
import edu.uci.ics.jung.visualization.picking.PickedState;
import edu.uci.ics.jung.visualization.GraphZoomScrollPane;
import edu.uci.ics.jung.visualization.control.ModalGraphMouse.Mode;
import edu.uci.ics.jung.visualization.decorators.ToStringLabeller;
import javax.swing.JList;
import java.awt.Font;
import org.apache.commons.collections15.functors.ConstantTransformer;

import java.util.Set;
import java.util.TreeSet;
import java.util.Vector;
import java.util.Iterator;

import java.util.Hashtable;

/**
 * 
 * This class represents the bigraph
 * 
 * @author Martin Schwartz
 *
 */
public class BiGraph 
{
	private UndirectedSparseGraph<VertexObj, EdgeObj> graph;
	private FRLayout<VertexObj, EdgeObj> layout;
	private VisualizationViewer<VertexObj, EdgeObj> vv;
	private VertexFillColour fillColour;
	private EdgeStroke edgeStroke;
	private EdgeFill edgeFill;
	private BiCliqueWin win;
	private VertexSizeShape vSize;
	private VertexToolTip vToolTip;
	private VertexStroke vStroke;
	private CustomPopupPlugin popupPlugin;

	/**
	 * @param w pointer to the GUI window
	 * @param d the dimensions of the panel which will contain the graph
	 */
	public BiGraph(BiCliqueWin w, Dimension d) 
	{
		d.setSize(d.getWidth()-20.0, d.getHeight()-20.0);
		win = w;
		graph = new UndirectedSparseGraph<VertexObj, EdgeObj>();
		vSize = new VertexSizeShape(this);
		vToolTip = new VertexToolTip();
		layout = new FRLayout<VertexObj, EdgeObj>(graph, d);
		layout.setMaxIterations(1000);
		vv = new VisualizationViewer<VertexObj, EdgeObj>(layout, d);
		vv.setBackground(Color.white);
		final CustomMouse mouse = new CustomMouse(win);
		mouse.setMode(Mode.PICKING);
		vv.setGraphMouse(mouse);
		vv.addKeyListener(mouse.getModeKeyListener());
		PickedState<VertexObj> picked_state = vv.getPickedVertexState();
		PickedState<EdgeObj> picked_edge_state = vv.getPickedEdgeState();
		fillColour = new VertexFillColour(picked_state);
		vStroke = new VertexStroke(picked_state);
		edgeStroke = new EdgeStroke(picked_edge_state);
		edgeFill = new EdgeFill();
		vv.getRenderContext().setVertexFillPaintTransformer(fillColour);
		vv.getRenderContext().setEdgeStrokeTransformer(edgeStroke);
		vv.getRenderContext().setEdgeDrawPaintTransformer(edgeFill);
		vv.getRenderContext().setVertexShapeTransformer(vSize);
		vv.getRenderContext().setVertexStrokeTransformer(vStroke);
		vv.setVertexToolTipTransformer(vToolTip);
		popupPlugin = new CustomPopupPlugin();
		mouse.add(popupPlugin);
		vv.getRenderContext().setVertexLabelTransformer(new ToStringLabeller<VertexObj>());
		vv.getRenderer().setVertexLabelRenderer(new CustomVertexLabelRenderer());
		//vv.getRenderer().getVertexLabelRenderer().setPosition(Renderer.VertexLabel.Position.S);
	}
	
	/**
	 * @return the visualisation viewer
	 */
	public VisualizationViewer<VertexObj, EdgeObj> getVV()
	{
		return vv;
	}
	
	/**
	 * @return the info file hash table
	 */
	public Hashtable<IDNodeTypePair, VertexObj> getInfoFile()
	{
		return win.getInfoFile().getIDToInfo();
	}
	
	/**
	 * clears the graph
	 */
	public void clearGraph()
	{
		changeGraph(new UndirectedSparseGraph<VertexObj, EdgeObj>());
		win.update(win.getGraphics());
	}
	
	/**
	 * @return the zoom scrollpane
	 */
	public GraphZoomScrollPane getZoomPane()
	{
		return new GraphZoomScrollPane(vv);
	}
	
	/**
	 * @return the graph object
	 */
	public UndirectedSparseGraph<VertexObj, EdgeObj> getGraph()
	{
		return graph;
	}
	
	/**
	 * @return the graph layout
	 */
	public FRLayout<VertexObj, EdgeObj> getGraphLayout()
	{
		return layout;
	}
	
	/**
	 * @return the selected vertices in the graph
	 */
	public Set<VertexObj> getPickedVertices()
	{
		return vv.getPickedVertexState().getPicked();
	}
	
	/**
	 * @param set the set of selected vertices
	 * @return the list of all communities that contain all selected vertices in them
	 */
	public JList findCommunities(Set<VertexObj> set)
	{
		TreeSet<ListCommunity> comms = new TreeSet<ListCommunity>(new CommunityComparer());
		
		VertexObj[] vs = set.toArray(new VertexObj[0]);
		
		Vector<TreeSet<Community>> commTree = win.getTreeMenu().getCommunitiesTree();

		for(int i = 0; i < commTree.size(); i++)
		{
			Vector<ListCommunity> ccs = new Vector<ListCommunity>();
			
			Iterator<Community> it = commTree.get(i).iterator();
			
			while(it.hasNext())
			{
				Community cc = it.next();
				ListCommunity comm = new ListCommunity(cc.getBicliques(), cc.getID(), cc.getCommType());
				
				boolean contains = true;
				boolean exist = false;
				
				Vector<Biclique> bcs = comm.getBicliques();
				
				for(int j = 0; j < vs.length; j++)
				{
					exist = false;
					
					for(int kk = 0; kk < bcs.size(); kk++)
					{
						if(vs[j].isVertexA())
						{
							for(int k = 0; k < bcs.get(kk).getBicliqueUpper().length; k++)
							{
								if(vs[j].getID() == bcs.get(kk).getBicliqueUpper()[k])
								{
									exist = true;
									break;
								}
							}
						}
						else
						{
							for(int k = 0; k < bcs.get(kk).getBicliqueLower().length; k++)
							{
								if(vs[j].getID() == bcs.get(kk).getBicliqueLower()[k])
								{
									exist = true;
									break;
								}
							}
						}
					}
					
					if(!(contains && exist))
					{
						break;
					}
				}
				
				if(contains && exist)
				{
					ccs.add(comm);
				}
			}
			
			if(ccs.size() > 0)
			{
				comms.addAll(ccs);
			}
		}
		
		JList communities = new JList(comms.toArray(new ListCommunity[0]));
		
		return communities;
	}
	
	/**
	 * @param s size for the labels
	 */
	@SuppressWarnings("unchecked")
	public void setLabelSize(int s)
	{
		vv.getRenderContext().setVertexFontTransformer(new ConstantTransformer(new Font("Helvetica", Font.PLAIN, s)));
	}
	
	/**
	 * @param b boolean for if scaling should be set or not
	 */
	public void setScaling(boolean b)
	{
		vSize.setScaling(b);
	}
	
	/**
	 * @param d the size for the scaling
	 */
	public void setScaleSize(int d)
	{
		vSize.setScaleSize(d);
	}
	
	/**
	 * @param g part-graph to be selected
	 */
	public void selectPartOfGraph(UndirectedSparseGraph<VertexObj, EdgeObj> g)
	{
		Iterator<EdgeObj> it = g.getEdges().iterator();
		
		while(it.hasNext())
		{
			EdgeObj e = it.next();
			vv.getPickedEdgeState().pick(e, true);
		}
		
		Iterator<VertexObj> it2 = g.getVertices().iterator();
		
		while(it2.hasNext())
		{
			VertexObj v = it2.next();
			vv.getPickedVertexState().pick(v, true);
		}
	}
	
	/**
	 * @param bs biclique to be converted to a graph object
	 * @return a graph object of the biclique
	 */
	public UndirectedSparseGraph<VertexObj, EdgeObj> createBiclique(Vector<Biclique> bs)
	{
		UndirectedSparseGraph<VertexObj, EdgeObj> g = new UndirectedSparseGraph<VertexObj, EdgeObj>();
		
		for(int k = 0; k < bs.size(); k++)
		{
			for(int i = 0; i < bs.get(k).getBicliqueUpper().length; i++)
			{
				for(int j = 0; j < bs.get(k).getBicliqueLower().length; j++)
				{
					int a = bs.get(k).getBicliqueUpper()[i];
					int b = bs.get(k).getBicliqueLower()[j];
					
					IDNodeTypePair pa = new IDNodeTypePair(a,true);
					IDNodeTypePair pb = new IDNodeTypePair(b,false);
					
					VertexObj va = getInfoFile().get(pa);
					VertexObj vb = getInfoFile().get(pb);
					
					if(va == null)
					{
						va = new VertexObj(null, a, true, null, "");
					}
					if(vb == null)
					{
						vb = new VertexObj(null, b, false, null, "");
					}
					
					EdgeObj e = new EdgeObj(va.getName(),vb.getName(), a, b);
					
					Pair<VertexObj> p = new Pair<VertexObj>(va, vb);
					g.addEdge(e, p);
				}
			}
		}
		
		return g;
	}
	
	/**
	 * @param g the new graph to substitute the old one
	 */
	public void changeGraph(Graph<VertexObj, EdgeObj> g)
	{
		graph = (UndirectedSparseGraph<VertexObj, EdgeObj>)g;
		layout.setGraph(graph);
		
		win.update(win.getGraphics());
	}
	
	/**
	 * Sets locations of the vertices in the graph
	 * @param numOfA number of Node A
	 * @param numOfB number of Node B
	 */
	public void setVertexLocations(int numOfA, int numOfB)
	{
		Iterator<VertexObj> it = graph.getVertices().iterator();
		
		double w = win.getMiddlePanel().getSize().getWidth();
		double h = win.getMiddlePanel().getSize().getHeight();
		
		final double OFFSET_X = 80;
		final double OFFSET_X_END = 40;
		final double NODE_A_OFFSET_Y = OFFSET_X;
		final double NODE_B_OFFSET_Y = h-OFFSET_X*2.0;
		
		double inc_a = (w-OFFSET_X-OFFSET_X_END)/(double)numOfA;
		double inc_b = (w-OFFSET_X-OFFSET_X_END)/(double)numOfB;
		
		double cur_a_offset = OFFSET_X;
		double cur_b_offset = OFFSET_X;
		while(it.hasNext())
		{
			VertexObj v = it.next();
			if(v.isVertexA())
			{
				layout.setLocation(v, cur_a_offset, NODE_A_OFFSET_Y);
				cur_a_offset += inc_a;
			}
			else
			{
				layout.setLocation(v, cur_b_offset, NODE_B_OFFSET_Y);
				cur_b_offset += inc_b;
			}
		}
	}
}
