package bicliqueviz;

import java.io.File;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.TreeMap;
import java.util.Vector;
import java.util.Iterator;
import java.util.TreeSet;

/**
 * 
 * This class manages the cache files of a bigraph to optimize the initialisation
 * after first creation of the bigraph
 * 
 * @author Martin Schwartz
 *
 */
public class CacheFilesSaver 
{
	private File cacheDir;
	//private BicliqueTreeMenu treeMenu;
	private Vector<String> children;
	private Vector<TreeSet<Community>> communityTree;
	private Vector<Biclique> bs;
	private InfoFile info;
	private final String COMM_BICLIQUES = "communities_bicliques";
	private final String VERTICES = "vertices";
	private final String BICLIQUES = "bicliques";


	/**
	 * @param f name of the graph file
	 * @param tm pointer to the tree menu
	 * @param g pointer to the bigraph
	 */
	public CacheFilesSaver(String f, Vector<String> c, Vector<TreeSet<Community>> ct, Vector<Biclique> bbs, InfoFile g) 
	{
		cacheDir =  new File(f+"_files");
		children = c;
		communityTree = ct;
		bs = bbs;
		info = g;
		if(!doesCacheFilesExist())
		{
			cacheDir.mkdir();
		}
	}
	
	/**
	 * Creates the cache files
	 */
	public void createCacheFiles()
	{	
		createCommDirs();
		createCommFiles();
		
		copyBicliqueFile();
		createVerticesFile();
	}
	
	private void createCommDirs()
	{
		for(int i = 0; i < children.size(); i++)
		{
			File dir = new File(cacheDir,children.get(i));
			if(!dir.exists())
			{
				dir.mkdir();
			}
		}
	}
	
	private void createCommFiles()
	{
		File[] files = cacheDir.listFiles();
		
		TreeMap<NumberPair, File> filesTree = new TreeMap<NumberPair, File>(new NumberPairComparer());
		
		for(int i = 0; i < files.length; i++)
		{
			if(files[i].isDirectory())
			{
				String s = files[i].getName();
				s = s.replace("k=", "");
				String[] split = s.split(",");
				NumberPair np = new NumberPair(Integer.parseInt(split[0]), Integer.parseInt(split[1]));
				filesTree.put(np, files[i]);
			}
		}
		
		Iterator<NumberPair> it2 = filesTree.keySet().iterator();
		
		int i = 0;
		while(it2.hasNext())
		{
			NumberPair np = it2.next();
			File d = filesTree.get(np);
			File commFile = new File(d,COMM_BICLIQUES);

			try 
			{
				BufferedWriter writer = new BufferedWriter(new FileWriter(commFile));
				
				TreeSet<Community> comms = communityTree.get(i);
				
				Iterator<Community> it = comms.iterator();
				
				while(it.hasNext())
				{
					Community c = it.next();
					writer.write(c.getID()+": ");
					
					Vector<Biclique> bs = c.getBicliques();
					
					for(int k = 0; k < bs.size(); k++)
					{
						writer.write(bs.get(k).getID()+" ");
					}
					writer.write("\n");
				}
				
				writer.close();
			} 
			catch (IOException e) 
			{
				e.printStackTrace();
			}
			i++;
		}
	}
	
	private void createVerticesFile()
	{
		try 
		{
			Iterator<IDNodeTypePair> it = info.getIDToInfo().keySet().iterator();
			BufferedWriter writer = new BufferedWriter(new FileWriter(cacheDir.getPath()+"/"+VERTICES));
			
			while(it.hasNext())
			{
				IDNodeTypePair p = it.next();
				VertexObj v = info.getIDToInfo().get(p);
				if(info.getIDToInfo().get(p).getNumOfCliqueOccurs() > 0)
				{
					writer.write(v.getID()+"\t"+v.getName()+"\t"+v.isVertexA()+"\t"+
							v.getNumOfCliqueOccurs()+"\t"+
							v.getColor().getRed()+","+v.getColor().getGreen()+","+
							v.getColor().getBlue()+"\t"+v.getShape()+"\n");
				}
			}
			writer.close();
		} 
		catch (IOException e) 
		{
			e.printStackTrace();
		}
	}
	
	private void copyBicliqueFile()
	{
		try 
		{
			BufferedWriter writer = new BufferedWriter(new FileWriter(cacheDir.getPath()+"/"+BICLIQUES));
			
			for(int i = 0; i < bs.size(); i++)
			{
				writer.write(i+": ");
				for(int j = 0; j < bs.get(i).getBicliqueUpper().length; j++)
				{
					writer.write(bs.get(i).getBicliqueUpper()[j]+" ");
				}
				writer.write("\n");
				
				writer.write(i+": ");
				for(int j = 0; j < bs.get(i).getBicliqueLower().length; j++)
				{
					writer.write(bs.get(i).getBicliqueLower()[j]+" ");
				}
				writer.write("\n");
			}
			writer.close();
		} 
		catch (IOException e) 
		{
			e.printStackTrace();
		}
	}
	
	/**
	 * @return true if cache directory exists, otherwise false
	 */
	public boolean doesCacheFilesExist()
	{
		return cacheDir.exists();
	}

}
