package bicliqueviz;

import java.util.Vector;
import java.util.HashSet;

/**
 * 
 * This class represents a given community in a bipartite graph
 * It contains information about the maximal bicliques within it.
 * 
 * @author Martin Schwartz
 *
 */
public class Community 
{
	protected int id;
	protected Vector<Biclique> bicliques = new Vector<Biclique>();
	protected String commType;
	private Integer numOfUpperNodes = null;
	private Integer numOfLowerNodes = null;
	private Integer numOfTotalNodes = null;

	/**
	 * @param b vector of maximal bicliques within the community
	 * @param i ID of the community
	 */
	public Community(Vector<Biclique> b, int i, String ct) 
	{
		bicliques = b;
		id = i;
		commType = ct;
	}
	
	/**
	 * @return the type of the community e.g. k=2,1
	 */
	public String getCommType()
	{
		return commType;
	}
	
	/**
	 * @return ID of the community
	 */
	public int getID()
	{
		return id;
	}
	
	/**
	 * @return vector of maximal bicliques within the community 
	 */
	public Vector<Biclique> getBicliques()
	{
		return bicliques;
	}
	
	/**
	 * @return a set of all upper nodes
	 */
	public HashSet<Integer> getAllUpperNodes()
	{
		HashSet<Integer> output = new HashSet<Integer>();
		
		for(int i = 0; i < bicliques.size(); i++)
		{
			for(int j = 0; j < bicliques.get(i).getBicliqueUpper().length; j++)
			{
				output.add(bicliques.get(i).getBicliqueUpper()[j]);
			}
		}
		
		return output;
	}
	
	/**
	 * @return the number of upper nodes in the community
	 */
	public Integer getNumOfUpperNodes()
	{
		if(numOfUpperNodes == null)
		{
			numOfUpperNodes = getAllUpperNodes().size();
		}
		
		return numOfUpperNodes;
	}
	
	/**
	 * @return a set of all lower nodes
	 */
	public HashSet<Integer> getAllLowerNodes()
	{
		HashSet<Integer> output = new HashSet<Integer>();
		
		for(int i = 0; i < bicliques.size(); i++)
		{
			for(int j = 0; j < bicliques.get(i).getBicliqueLower().length; j++)
			{
				output.add(bicliques.get(i).getBicliqueLower()[j]);
			}
		}
		
		return output;
	}
	
	/**
	 * @return the numbder of lower nodes in the community
	 */
	public Integer getNumOfLowerNodes()
	{
		if(numOfLowerNodes == null)
		{
			numOfLowerNodes = getAllLowerNodes().size();
		}
		
		return numOfLowerNodes;
	}
	
	/**
	 * @return the number of total nodes (both upper and lower) in the community
	 */
	public Integer getNumOfTotalNodes()
	{
		if(numOfTotalNodes == null)
		{
			numOfTotalNodes = getNumOfUpperNodes() + getNumOfLowerNodes();
		}
		
		return numOfTotalNodes;
	}
	
	public NumberPair getNumOfOverlappingNodes(Community c)
	{
		int upperNodes = 0;
		int lowerNodes = 0;
		HashSet<Integer> un = getAllUpperNodes();
		HashSet<Integer> ln = getAllLowerNodes();
		
		HashSet<Integer> un2 = c.getAllUpperNodes();
		HashSet<Integer> ln2 = c.getAllLowerNodes();
		
		Integer[] uns = un2.toArray(new Integer[0]);
		
		for(int i = 0; i < uns.length; i++)
		{
			if(un.contains(uns[i]))
			{
				upperNodes++;
			}
		}
		
		Integer[] lns = ln2.toArray(new Integer[0]);
		
		for(int i = 0; i < lns.length; i++)
		{
			if(ln.contains(lns[i]))
			{
				lowerNodes++;
			}
		}
		
		return new NumberPair(upperNodes,lowerNodes);
	}
	
	public HashSet<Integer>[] getOverlappingNodes(Community c)
	{
		HashSet<Integer>[] overlap = new HashSet[2];
		overlap[0] = new HashSet<Integer>();
		overlap[1] = new HashSet<Integer>();
		HashSet<Integer> un = getAllUpperNodes();
		HashSet<Integer> ln = getAllLowerNodes();
		
		HashSet<Integer> un2 = c.getAllUpperNodes();
		HashSet<Integer> ln2 = c.getAllLowerNodes();
		
		Integer[] uns = un2.toArray(new Integer[0]);
		
		for(int i = 0; i < uns.length; i++)
		{
			if(un.contains(uns[i]))
			{
				overlap[0].add(uns[i]);
			}
		}
		
		Integer[] lns = ln2.toArray(new Integer[0]);
		
		for(int i = 0; i < lns.length; i++)
		{
			if(ln.contains(lns[i]))
			{
				overlap[1].add(lns[i]);
			}
		}
		
		return overlap;
	}
	
	/**
	 * Increment the vertex occurences
	 * @param info the info files
	 */
	public void incrementVertexOccurs(InfoFile info)
	{
		HashSet<IDNodeTypePair> set = new HashSet<IDNodeTypePair>();
		
		for(int i = 0; i < bicliques.size(); i++)
		{
			Biclique b = bicliques.get(i);
			
			for(int k = 0; k < b.getBicliqueUpper().length; k++)
			{
				IDNodeTypePair p = new IDNodeTypePair(b.getBicliqueUpper()[k], true);
				if(!set.contains(p))
				{
					set.add(p);
					info.setNodeIDToOccurs(p);
				}
			}
			
			for(int k = 0; k <  b.getBicliqueLower().length; k++)
			{
				IDNodeTypePair p = new IDNodeTypePair(b.getBicliqueLower()[k], false);
				if(!set.contains(p))
				{
					set.add(p);
					info.setNodeIDToOccurs(p);
				}
			}
		}
	}
	
	public String toString()
	{
		return id +". comm. (A: "+ getNumOfUpperNodes() +" ,B: "+getNumOfLowerNodes()+ ")";
	}

}
