package bicliqueviz;

import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.ListSelectionModel;

import java.awt.Component;
import javax.swing.BoxLayout;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;
import javax.swing.JList;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.File;
import java.io.IOException;
import java.util.Vector;
import javax.swing.text.Position;
import javax.swing.JScrollPane;
import javax.swing.JPopupMenu;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * 
 * This class represents tabbed panes in the GUI
 * 
 * @author Martin Schwartz
 *
 */
public class TabbedPane extends MouseAdapter implements ChangeListener, ListSelectionListener, ActionListener
{
	private JTabbedPane tabs;
	private BicliqueTreeMenu treeMenu;
	private BiGraph graph;
	private TabbedPane otherPane;
	private VertexCommList vcList;
	private BiCliqueWin win;
	private JPopupMenu popupMenu;
	private final String LINKS = "links";
	private Object selectedPopup;

	/**
	 * @param w window instance
	 */
	public TabbedPane(BiCliqueWin w) 
	{
		tabs = new JTabbedPane();
		tabs.addChangeListener(this);
		win = w;
		popupMenu = createPopupMenu();
	}
	
	/**
	 * @param title title of the tab
	 * @param left vertex list
	 * @param middle edges list
	 * @param right biclique list
	 * @param g graph instance
	 */
	public void addTab(String title, JList left, 
			JList middle, JList right, BiGraph g)
	{
		left.addListSelectionListener(this);
		left.addMouseListener(this);
		left.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		middle.addListSelectionListener(this);
		middle.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		right.addListSelectionListener(this);
		right.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		graph = g;
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
		panel.add(new JScrollPane(left));
		panel.add(new JScrollPane(middle));
		panel.add(new JScrollPane(right));
		tabs.add(title, panel);
		setSelectedTab(panel);
	}
	
	private JPopupMenu createPopupMenu()
	{
		JPopupMenu pMenu = new JPopupMenu();
		
		try 
		{
			File linksFile = new File(LINKS);
			
			if(!linksFile.exists())
			{
				linksFile.createNewFile();
			}
			BufferedReader reader = new BufferedReader(new FileReader(linksFile));
			
			String line = reader.readLine();
			
			while(line != null)
			{
				String[] splitLine = line.split("\t");
				ListPopupItem lpi = new ListPopupItem(splitLine[0], splitLine[1]);
				lpi.addActionListener(this);
				pMenu.add(lpi);
				line = reader.readLine();
			}
			reader.close();
		} 
		catch (FileNotFoundException e) 
		{
			e.printStackTrace();
		}
		catch (IOException e) 
		{
			e.printStackTrace();
		}
		
		return pMenu;
	}
	
	public void actionPerformed(ActionEvent ae)
	{
		if (ae.getSource().getClass() == ListPopupItem.class)
		{
			ListPopupItem lpi = (ListPopupItem)ae.getSource();
			
			if(selectedPopup != null)
			{
				lpi.runWebSearch(selectedPopup.toString());
			}
		}
	}
	
	final public void mousePressed(MouseEvent mouseEvent)
    {
        handleEvent(mouseEvent);
    }
    
    final public void mouseReleased(MouseEvent mouseEvent)
    {
        handleEvent(mouseEvent);
    }
    
    final public void mouseClicked(MouseEvent mouseEvent)
    {
        handleEvent(mouseEvent);
    }
	
	private void handleEvent(MouseEvent mouseEvent)
    {
        if (mouseEvent.isPopupTrigger())
        {
            processPopup(mouseEvent);
        }
    }
	
	private void processPopup(MouseEvent me)
	{
		JList list = (JList)me.getComponent();
		
		if(!list.isSelectionEmpty())
		{
			int[] indices = list.getSelectedIndices();
			Object[] values = list.getSelectedValues();
			
			boolean found = false;
			
			for(int i = 0; i < indices.length; i++)
			{
				if(list.locationToIndex(me.getPoint()) == indices[i])
				{
					found = true;
					selectedPopup = values[i];
					break;
				}
			}
			
			if(found)
			{
				popupMenu.show(list, me.getX(), me.getY());
			}
			
		}
	}
	
	/**
	 * @return bicliques list of selected tab
	 */
	public JList getBicliquesList()
	{
		JPanel jp = (JPanel)tabs.getSelectedComponent();
		if(jp != null)
		{
			JScrollPane jsp = (JScrollPane)jp.getComponent(2);
			return (JList)jsp.getViewport().getComponent(0);
		}
		else
		{
			return null;
		}
	}

	/**
	 * @return edges list of selected tab
	 */
	public JList getEdgesList()
	{
		JPanel jp = (JPanel)tabs.getSelectedComponent();
		JScrollPane jsp = (JScrollPane)jp.getComponent(1);
		return (JList)jsp.getViewport().getComponent(0);
	}
	
	/**
	 * @return vertex list of selected tab
	 */
	public JList getVertexList()
	{
		JPanel jp = (JPanel)tabs.getSelectedComponent();
		JScrollPane jsp = (JScrollPane)jp.getComponent(0);
		return (JList)jsp.getViewport().getComponent(0);
	}
	
	/**
	 * 
	 * Sets other tabbed pane
	 * 
	 * @param t pointer to other tabbed pane
	 */
	public void setOtherTabbedPane(TabbedPane t)
	{
		otherPane = t;
	}
	
	public void setTreeMenu(BicliqueTreeMenu tm)
	{
		treeMenu = tm;
	}
	
	/**
	 * 
	 * Sets vertex community list
	 * 
	 * @param vcl pointer to vertex community list
	 */
	public void setVertexCommList(VertexCommList vcl)
	{
		vcList = vcl;
	}
	
	/**
	 * 
	 * Set selected tab
	 * 
	 * @param c component to be selected
	 */
	private void setSelectedTab(Component c)
	{
		tabs.setSelectedComponent(c);
	}
	
	/**
	 * @return the tabbed pane
	 */
	public JTabbedPane getTabbedPane()
	{
		return tabs;
	}
	
	/**
	 * @param t title of tab to be removed
	 */
	public void removeTab(String t)
	{
		for(int i = 0; i < tabs.getTabCount(); i++)
		{
			if(t.equals(tabs.getTitleAt(i)))
			{
				tabs.remove(i);
				break;
			}
		}
	}
	
	public void valueChanged(ListSelectionEvent e) 
	{
		JList list = (JList)e.getSource();
		int category = -1;
		Object[] values = list.getSelectedValues();
		
		if(e.getValueIsAdjusting())
		{
			if(list.getModel().getElementAt(0).getClass().equals(VertexObj.class))
			{
				VertexObj v = (VertexObj)list.getModel().getElementAt(0);
				
				if(v.isVertexA())
				{
					category = 0;
				}
				else
				{
					category = 1;
				}
			}
			else if(list.getModel().getElementAt(0).getClass().equals(Biclique.class))
			{
				if(values.length == 0)
				{
					otherPane.getBicliquesList().clearSelection();
				}
				else
				{
					otherPane.getBicliquesList().setSelectedIndices(getBicliquesList().getSelectedIndices());
				}
				category = 2;
			}
			else
			{
				if(values.length == 0)
				{
					otherPane.getEdgesList().clearSelection();
				}
				else
				{
					otherPane.getEdgesList().setSelectedIndices(getEdgesList().getSelectedIndices());
				}
			}
			
			Object[] selected;
			if(category == 0 || category == 1)
			{
				selected = graph.getVV().getPickedVertexState().getSelectedObjects();
			}
			else
			{
				selected = graph.getVV().getPickedEdgeState().getSelectedObjects();
			}
			
			Vector<VertexObj> aNodes = new Vector<VertexObj>();
			Vector<VertexObj> bNodes = new Vector<VertexObj>();
			Vector<EdgeObj> edges = new Vector<EdgeObj>();
			
			for(int i = 0; i < selected.length; i++)
			{
				if(selected[i].getClass().equals(VertexObj.class))
				{
					VertexObj v = (VertexObj)selected[i];
					
					if(v.isVertexA())
					{
						aNodes.add(v);
					}
					else
					{
						bNodes.add(v);
					}
				}
				else if(selected[i].getClass().equals(EdgeObj.class))
				{
					EdgeObj ed = (EdgeObj)selected[i];
					edges.add(ed);
				}
			}
			
			//Vertex A selected from list
			if(category == 0)
			{
				if(getBicliquesList().getSelectedValues().length > 0)
				{
					win.clearBicliqueList();
					win.clearGraphSelection();
				}
				if(values.length > aNodes.size())
				{
					for(int i = 0; i < values.length; i++)
					{		
						VertexObj v = (VertexObj)values[i];
						graph.getVV().getPickedVertexState().pick(v, true);
					}
				}
				else
				{
					for(int i = 0; i < aNodes.size(); i++)
					{
						boolean notThere = true;
						
						for(int j = 0; j < values.length; j++)
						{
							if(aNodes.get(i).equals(values[j]))
							{
								notThere = false;
								break;
							}
						}
						if(notThere)
						{
							VertexObj v = aNodes.get(i);
							graph.getVV().getPickedVertexState().pick(v, false);
							break;
						}
					}
					for(int i = 0; i < values.length; i++)
					{		
						VertexObj v = (VertexObj)values[i];
						graph.getVV().getPickedVertexState().pick(v, true);
					}
				}
			}
			//Vertex B selected from list
			else if(category == 1)
			{
				if(getBicliquesList().getSelectedValues().length > 0)
				{
					win.clearBicliqueList();
					win.clearGraphSelection();
				}
				if(values.length > bNodes.size())
				{
					for(int i = 0; i < values.length; i++)
					{		
						VertexObj v = (VertexObj)values[i];
						graph.getVV().getPickedVertexState().pick(v, true);
					}
				}
				else
				{
					for(int i = 0; i < bNodes.size(); i++)
					{
						boolean notThere = true;
						
						for(int j = 0; j < values.length; j++)
						{
							if(bNodes.get(i).equals(values[j]))
							{
								notThere = false;
								break;
							}
						}
						if(notThere)
						{
							VertexObj v = bNodes.get(i);
							graph.getVV().getRenderContext().getPickedVertexState().pick(v, false);
							break;
						}
					}
					for(int i = 0; i < values.length; i++)
					{		
						VertexObj v = (VertexObj)values[i];
						graph.getVV().getPickedVertexState().pick(v, true);
					}
				}
			}
			//Biclique selected from list
			else if(category == 2)
			{
				Vector<Biclique> b = new Vector<Biclique>();
				win.clearListSelection();
				win.clearGraphSelection();
				for(int i = 0; i < values.length; i++)
				{
					b.add((Biclique)values[i]);
				}
				graph.selectPartOfGraph(graph.createBiclique(b));
			}
			//Edge selected from list
			else
			{
				if(getBicliquesList().getSelectedValues().length > 0)
				{
					win.clearBicliqueList();
					win.clearGraphSelection();
				}
				if(values.length > edges.size())
				{
					for(int i = 0; i < values.length; i++)
					{		
						EdgeObj ed = (EdgeObj)values[i];
						graph.getVV().getPickedEdgeState().pick(ed, true);
					}
				}
				else
				{
					for(int i = 0; i < edges.size(); i++)
					{
						boolean notThere = true;
						
						for(int j = 0; j < values.length; j++)
						{
							if(edges.get(i).equals(values[j]))
							{
								notThere = false;
								break;
							}
						}
						if(notThere)
						{
							EdgeObj ed = edges.get(i);
							graph.getVV().getPickedEdgeState().pick(ed, false);
							break;
						}
					}
					for(int i = 0; i < values.length; i++)
					{		
						EdgeObj ed = (EdgeObj)values[i];
						graph.getVV().getPickedEdgeState().pick(ed, true);
					}
				}
			}
		}
	}
	
	private void clearAllSelections()
	{
		getVertexList().clearSelection();
		getEdgesList().clearSelection();
		getBicliquesList().clearSelection();
	}
	
	public void stateChanged(ChangeEvent e)
	{	
		if(tabs.getSelectedIndex() != -1)
		{
			String t = tabs.getTitleAt(tabs.getSelectedIndex());
			String[] s = t.split("\\s", 2);
			
	    	TreePath[] paths = treeMenu.getJTree().getSelectionPaths();
	    	
	    	clearAllSelections();
	    	
//	    	selectEdges(graph.getVV().getPickedEdgeState().getPicked());
//	    	selectVertices(graph.getVV().getPickedVertexState().getPicked());
	    	
	    	if(tabs.getTabCount() == otherPane.getTabbedPane().getTabCount())
	    	{
	    		otherPane.getTabbedPane().setSelectedIndex(tabs.getSelectedIndex());
	    	}
	    	
	    	if(t.contains("community"))
	    	{
	    		String title = tabs.getTitleAt(tabs.getSelectedIndex());
	    		int i = vcList.getJList().getNextMatch(title, 0, Position.Bias.Forward);
	    		vcList.getJList().setSelectedIndex(i);
	    		Community c = (Community)vcList.getJList().getSelectedValue();
	    		
	    		graph.changeGraph(graph.createBiclique(c.getBicliques()));
	    	}
	    	else
	    	{
	    		vcList.getJList().clearSelection();
		    	for(int j = 0; j < paths.length; j++)
		    	{
		    		DefaultMutableTreeNode node = 
						(DefaultMutableTreeNode)paths[j].getLastPathComponent();
		
		    		s[1] = s[1].replace("(", "");
		    		s[1] = s[1].replace(")", "");
		    		if(s[1].equals(node.getParent().toString()))
		    		{
		    			s[0] = s[0].trim();
		    			if(s[0].equals(node.toString().split("\\s")[0]))
		    			{
		    				Community comm = (Community)node.getUserObject();
							graph.changeGraph(graph.createBiclique(comm.getBicliques()));
		    				break;
		    			}
		    		}
		    	}
	    	}
		}
	}

}
